"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var nock_1 = require("nock");
var path_1 = __importDefault(require("path"));
var lodash_1 = require("lodash");
var TEST_MODE = process.env.TEST_MODE;
var overrideMethods = [
    "it",
    "fit",
    "xit",
    "beforeAll",
    "beforeEach",
    "afterAll",
    "afterEach"
];
var optionDefaults = {
    fixtureDir: "test/fixtures",
    jasmine: null,
    global: null,
    defaultMode: "dryrun",
    generateMockName: generateMockName,
    nock: {
        before: before,
        after: after,
        afterRecord: afterRecord
    }
};
function JestNockBack(options) {
    var opts = Object.assign({}, optionDefaults, options);
    var jasmine = opts.jasmine, _global = opts.global, fixtureDir = opts.fixtureDir, defaultMode = opts.defaultMode;
    if (!jasmine || !_global) {
        throw new Error("Must provide `jasmine` and `global` as options to this function!");
    }
    var nockMode = TEST_MODE || defaultMode || "dryrun";
    // Resolve whatever path was given to us and turn it into absolute path
    nock_1.back.fixtures = opts.fixtureDir = path_1.default.resolve(fixtureDir);
    nock_1.back.setMode(nockMode);
    var env = jasmine.getEnv();
    var testPath = jasmine.testPath;
    overrideMethods.forEach(function (method) {
        _global[method].nock = bindNock(method, env[method], env, testPath, opts);
    });
}
exports.JestNockBack = JestNockBack;
function bindNock(method, fn, env, testPath, options) {
    return function test() {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var title = args[0], testFn = args[1], timeout = args[2];
        // Handle beforeAll, etc
        if (typeof title === "function") {
            testFn = title;
            title = method;
        }
        // Handle callback style tests
        if (testFn && testFn.length) {
            testFn = wrapTestFnCallbackWithPromise(testFn);
        }
        // Replace the fn w/ our wrapped one
        testFn = nockedTestFn(method, title, testFn, env, testPath, options);
        // Return the test
        return fn(title, testFn, timeout);
    };
}
function nockedTestFn(method, title, fn, env, testPath, options) {
    var _this = this;
    return function () { return __awaiter(_this, void 0, void 0, function () {
        var fixtureName, _a, nockDone, context, ret;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    fixtureName = options.generateMockName(title, testPath, options.fixtureDir);
                    return [4 /*yield*/, nock_1.back(fixtureName, options.nock)];
                case 1:
                    _a = _b.sent(), nockDone = _a.nockDone, context = _a.context;
                    ret = fn.call({ nock: context });
                    // If it wasn't a promise, this was a "syncronous"
                    // test, and we are done.
                    if (!isPromise(ret)) {
                        nockDone();
                        return [2 /*return*/, ret];
                    }
                    // Otherwise wait around for the test's
                    // Promise chain to finish up.
                    return [2 /*return*/, ret.finally(function () { return nockDone(); })];
            }
        });
    }); };
}
function generateMockName(testTitle, testPath, fixtureDirectory) {
    var shortPath = path_1.default.relative(fixtureDirectory, testPath);
    var testExt = path_1.default.extname(shortPath);
    var testPrefix = shortPath.replace("../", "").replace(testExt, "");
    return lodash_1.kebabCase(testPrefix + " " + testTitle) + ".json";
}
function after(scope) { }
function before(def) { }
function afterRecord(defs) {
    return defs;
}
function isPromise(val) {
    return val && typeof val.then === "function";
}
function wrapTestFnCallbackWithPromise(testFn) {
    return function () {
        return new Promise(function (resolve, reject) {
            return testFn(function (err) {
                if (err) {
                    return reject(err);
                }
                return resolve();
            });
        });
    };
}
